

local SpeakEasyGUI = SpeakEasyGUI;




function SpeakEasyGUI:Config_ToggleDisplay()
	local frame = SpeakEasyGUI_DisplayFrame;
	
	if (frame:IsShown()) then
		CloseDropDownMenus();
		
		frame:Hide();
		
		if (self.PopulateTableOnHide) then
			self.PopulateTableOnHide = nil;
			self:Config_PopulateDisplayTable();
		end
		
		
	else
		SpeakEasyGUI_RandomEditBoxesTitle:SetText(self.LOCALIZATION.MISC_EDIT_BOXES_TITLE:format(self.MIN_RANDOM, self.MAX_RANDOM));
		
		self:Config_LoadValuesForSelectedEntry();
		
		frame:Show();
	end
end




function SpeakEasyGUI:Config_SetSelectedEntry(catNum, actionNum, entryNum)
	if (catNum == nil) then catNum = (self.CurrentCategoryNumber or 1); end
	
	if (catNum == 4) then
		if (not next(self.LocalConfig.SelfBuffs)) then
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.COMMANDS_HELP_1, 0.0, 0.8, 1.0);
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.COMMANDS_HELP_4, 0.0, 0.8, 1.0);
			return nil;
		end
		
	elseif (catNum == 5) then
		if (not next(self.LocalConfig.SelfDebuffs)) then
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.COMMANDS_HELP_1, 0.0, 0.8, 1.0);
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.COMMANDS_HELP_5, 0.0, 0.8, 1.0);
			return nil;
		end
	end
	
	
	if (not actionNum) then actionNum = (self.CurrentActionNumber or 1); end
	if (not entryNum) then entryNum = (self.CurrentEntryNumber or 1); end
	
	
	local speechCategory = self.SpeechCategories[catNum];
	
	
	self.CurrentCategoryNumber = catNum;
	SpeakEasyGUI_CategoryDropDownText:SetText(speechCategory.name);
	
	if (speechCategory.showSendAtStart) then
		SpeakEasyGUI_SendAtStartCheckbox:Show();
	else
		SpeakEasyGUI_SendAtStartCheckbox:Hide();
	end
	
	
	self.CurrentActionNumber = actionNum;
	
	local actionName = speechCategory.subList[actionNum];
	SpeakEasyGUI_ActionDropDownText:SetText(
	  self.LOCALIZATION.COMBAT_EVENTS_LIST[actionName]
	  or self.LOCALIZATION.MISC_EVENTS_LIST[actionName]
	  or actionName
	);
	
	
	self.CurrentEntryNumber = entryNum;
	SpeakEasyGUI_EntryDropDownText:SetText(entryNum);
	
	
	self.StorageTablePointer = (self.SpeechCategories[catNum].storagePointer or self.LocalConfig);
end




function SpeakEasyGUI:Config_LoadValuesForSelectedEntry()
	local action = self.StorageTablePointer[self.SpeechCategories[self.CurrentCategoryNumber].subList[self.CurrentActionNumber]];
	
	local entry = nil;
	if (action) then
		entry = action[self.CurrentEntryNumber];
	end
	
	
	if (action and entry) then
		
		SpeakEasyGUI_DisallowRepeatCheckbox:SetChecked(entry[7]);
		SpeakEasyGUI_SendAtStartCheckbox:SetChecked(entry[8]);
		
		SpeakEasyGUI_MessageEditBox:SetText(entry[1]);
		
		SpeakEasyGUI_MinRandomEditBox:SetText(entry[2]);
		SpeakEasyGUI_MaxRandomEditBox:SetText(entry[3]);
		
		SpeakEasyGUI_MessageTypeDropDownText:SetText(self.MESSAGE_TYPE_LIST[entry[4]]);
		self.SelectedMessageType = entry[4];
		
		SpeakEasyGUI_ChannelOrWhisperEditBox:SetText(entry[6] or "");
		
		SpeakEasyGUI_TimeDelayEditBox:SetText(entry[10] or "");
		
		SpeakEasyGUI_LanguageDropDownText:SetText(self.LANGUAGE_LIST[entry[5]]);
		self.SelectedLanguage = entry[5];
		
	else
		
		SpeakEasyGUI_DisallowRepeatCheckbox:SetChecked(nil);
		SpeakEasyGUI_SendAtStartCheckbox:SetChecked(nil);
		
		SpeakEasyGUI_MessageEditBox:SetText("");
		
		SpeakEasyGUI_MinRandomEditBox:SetText("");
		SpeakEasyGUI_MaxRandomEditBox:SetText("");
		
		SpeakEasyGUI_MessageTypeDropDownText:SetText(CHAT_MSG_SAY);
		self.SelectedMessageType = 8;
		
		SpeakEasyGUI_ChannelOrWhisperEditBox:SetText("");
		
		SpeakEasyGUI_TimeDelayEditBox:SetText("");
		
		if (GetDefaultLanguage() == "Common") then
			SpeakEasyGUI_LanguageDropDownText:SetText("Common");
			self.SelectedLanguage = 1;
		else
			SpeakEasyGUI_LanguageDropDownText:SetText("Orcish");
			self.SelectedLanguage = 7;
		end
		
	end
end




function SpeakEasyGUI:Config_ValidateEditBoxes()
	if (self.STRING_LIBRARY.gsub(SpeakEasyGUI_MessageEditBox:GetText(), " ", "") == "") then
		DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ERROR_MESSAGE_BOX, 0.0, 0.8, 1.0);
		return nil;
	end
	
	
	local number = SpeakEasyGUI_MinRandomEditBox:GetNumber();
	if (number < self.MIN_RANDOM or number > self.MAX_RANDOM or number > SpeakEasyGUI_MaxRandomEditBox:GetNumber()) then
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.STRING_LIBRARY.format(self.LOCALIZATION.ERROR_MIN_NUMBER_BOX, self.MIN_RANDOM, self.MAX_RANDOM),
		  0.0, 0.8, 1.0
		);
		return nil;
	end
	
	
	number = SpeakEasyGUI_MaxRandomEditBox:GetNumber();
	if (number < self.MIN_RANDOM or number > self.MAX_RANDOM or number < SpeakEasyGUI_MinRandomEditBox:GetNumber()) then
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.STRING_LIBRARY.format(self.LOCALIZATION.ERROR_MAX_NUMBER_BOX, self.MIN_RANDOM, self.MAX_RANDOM),
		  0.0, 0.8, 1.0
		);
		return nil;
	end
	
	
	if (self.SelectedMessageType == 1 or self.SelectedMessageType == 9) then
		local text = self.STRING_LIBRARY.gsub(SpeakEasyGUI_ChannelOrWhisperEditBox:GetText(), "[^%w]+", "");
		if (text == "" or text == " ") then
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ERROR_CHANNEL_OR_WHISPER_BOX, 0.0, 0.8, 1.0);
			return nil;
		end
	end
	
	
	number = SpeakEasyGUI_TimeDelayEditBox:GetNumber();
	if (number == 0) then
		SpeakEasyGUI_TimeDelayEditBox:SetText("");
		
	elseif (number < self.MIN_DELAY or number > self.MAX_DELAY) then
		DEFAULT_CHAT_FRAME:AddMessage(
		  self.STRING_LIBRARY.format(self.LOCALIZATION.ERROR_TIME_DELAY_BOX, self.MIN_DELAY, self.MAX_DELAY),
		  0.0, 0.8, 1.0
		);
		return nil;
	end
	
	
	if (self.SelectedMessageType == 2) then
		local text = self.STRING_LIBRARY.upper(SpeakEasyGUI_MessageEditBox:GetText());
		local emote = getglobal("EMOTE"..text.."_TOKEN");
		if (emote) then
			SpeakEasyGUI_MessageEditBox:SetText(emote);
			
		else
			local numEmotes = self.NUM_EMOTES;
			local getglobal = getglobal;
			text = self.STRING_LIBRARY.upper(text);
			
			local index = 1;
			while (index <= numEmotes) do
				if (text == getglobal("EMOTE"..index.."_TOKEN")) then
					index = (numEmotes + 1);
				else
					index = (index + 1);
				end
			end
			
			if (index == (numEmotes + 1)) then
				SpeakEasyGUI_MessageEditBox:SetText(text);
			else
				DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.ERROR_INVALID_BASIC_EMOTE, 0.0, 0.8, 1.0);
				return nil;
			end
		end
	end
	
	
	return true;
end



function SpeakEasyGUI:Config_SaveValuesForSelectedEntry()
	local actionName = self.SpeechCategories[self.CurrentCategoryNumber].subList[self.CurrentActionNumber];
	
	local action = self.StorageTablePointer[actionName];
	
	
	if (not action) then
		self.StorageTablePointer[actionName] = {
			[self.CurrentEntryNumber] = {};
		};
		action = self.StorageTablePointer[actionName][self.CurrentEntryNumber];
		
	elseif (not action[self.CurrentEntryNumber]) then
		action[self.CurrentEntryNumber] = {};
		action = action[self.CurrentEntryNumber];
		
	else
		action = action[self.CurrentEntryNumber];
	end
	
	
	action[1] = SpeakEasyGUI_MessageEditBox:GetText();
	action[2] = SpeakEasyGUI_MinRandomEditBox:GetNumber();
	action[3] = SpeakEasyGUI_MaxRandomEditBox:GetNumber();
	
	
	local messageType = self.SelectedMessageType;
	action[4] = messageType;
	
	if (messageType == 1 or messageType == 2 or messageType == 9) then
		action[6] = SpeakEasyGUI_ChannelOrWhisperEditBox:GetText();
	else
		action[6] = nil;
	end
	
	
	action[5] = self.SelectedLanguage;
	
	
	if (SpeakEasyGUI_DisallowRepeatCheckbox:GetChecked()) then
		action[7] = true;
	else
		action[7] = nil;
	end
	
	
	if (SpeakEasyGUI_SendAtStartCheckbox:GetChecked()) then
		action[8] = true;
	else
		action[8] = nil;
	end
	
	
	local number = self.MATH_LIBRARY.ceil(SpeakEasyGUI_TimeDelayEditBox:GetNumber());
	if (number > 0) then
		action[10] = number;
	else
		action[10] = nil;
	end
	
	
	self:SpecialEntries_ManageEventRegistration();
end



SpeakEasyGUI_SaveChangesButton:SetScript("OnClick", function(self)
	local segui = self:GetParent().SPEAKEASYGUI;
	if (segui:Config_ValidateEditBoxes()) then
		segui:Config_SaveValuesForSelectedEntry();
		segui:Config_LoadValuesForSelectedEntry();
	end
end);




function SpeakEasyGUI:Config_ClearSelectedEntry()
	local actionName = self.SpeechCategories[self.CurrentCategoryNumber].subList[self.CurrentActionNumber];
	local action = self.StorageTablePointer[actionName];
	
	if (action) then
		action[self.CurrentEntryNumber] = nil;
		
		if (not next(action)) then
			self.StorageTablePointer[actionName] = nil;
		end
	end
	
	
	DEFAULT_CHAT_FRAME:AddMessage(self.STRING_LIBRARY.format(self.LOCALIZATION.FEEDBACK_ENTRY_CLEARED,
	  self.CurrentEntryNumber,
	  (self.LOCALIZATION.COMBAT_EVENTS_LIST[actionName] or self.LOCALIZATION.MISC_EVENTS_LIST[actionName] or actionName),
	  self.SpeechCategories[self.CurrentCategoryNumber].name),
	  0.0, 0.8, 1.0
	);
	
	self:Config_LoadValuesForSelectedEntry();
end


SpeakEasyGUI_ClearEntryButton:SetScript("OnClick", (function(self) self:GetParent().SPEAKEASYGUI:Config_ClearSelectedEntry(); end));




function SpeakEasyGUI:Config_EntryDropDown_OnClick()
	self:Config_SetSelectedEntry(nil, nil, this.value);
	self:Config_LoadValuesForSelectedEntry();
end



function SpeakEasyGUI:Config_EntryDropDown_Initialize()
	local buttonInfo = self:Main_GetReusableTable(1);
	buttonInfo.func = self.Config_EntryDropDown_OnClick;
	buttonInfo.arg1 = self;
	
	local addButton = UIDropDownMenu_AddButton;
	local action = self.StorageTablePointer[self.SpeechCategories[self.CurrentCategoryNumber].subList[self.CurrentActionNumber]];
	
	for index = 1, self.MAX_ENTRIES do
		buttonInfo.text = index;
		buttonInfo.value = index;
		buttonInfo.checked = (action ~= nil and action[index] ~= nil);
		addButton(buttonInfo);
	end
end


SpeakEasyGUI_EntryDropDown.initialize = (function() SpeakEasyGUI:Config_EntryDropDown_Initialize(); end);




function SpeakEasyGUI:Config_ActionDropDown_OnEnter()
	if (self.CurrentCategoryNumber == 1) then
		GameTooltip:SetOwner(SpeakEasyGUI_ActionDropDown, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(self.LOCALIZATION.HOTKEY_MACRO_TIP:format(self.CurrentEntryNumber, self.CurrentEntryNumber), 1.0, 1.0, 1.0);
	end
end


SpeakEasyGUI_ActionDropDown:SetScript("OnEnter", function() SpeakEasyGUI:Config_ActionDropDown_OnEnter(); end);



function SpeakEasyGUI:Config_ActionDropDown_OnClick()
	self:Config_SetSelectedEntry(nil, this.value, 1);
	self:Config_LoadValuesForSelectedEntry();
end



function SpeakEasyGUI:Config_ActionDropDown_Initialize()
	local buttonInfo = self:Main_GetReusableTable(1);
	buttonInfo.func = self.Config_ActionDropDown_OnClick;
	buttonInfo.arg1 = self;
	
	local combatEvents = self.LOCALIZATION.COMBAT_EVENTS_LIST;
	local miscEvents = self.LOCALIZATION.MISC_EVENTS_LIST;
	local config = self.StorageTablePointer;
	local addButton = UIDropDownMenu_AddButton;
	
	local tempName;
	for entryNum, spellName in ipairs(self.SpeechCategories[self.CurrentCategoryNumber].subList) do
		buttonInfo.value = entryNum;
		tempName = (combatEvents[spellName] or miscEvents[spellName] or spellName);
		buttonInfo.text = tempName;
		buttonInfo.checked = (config[tempName] ~= nil or config[spellName] ~= nil);
		addButton(buttonInfo);
	end
end


SpeakEasyGUI_ActionDropDown.initialize = (function() SpeakEasyGUI:Config_ActionDropDown_Initialize(); end);




function SpeakEasyGUI:Config_CategoryDropDown_OnClick(value)
	self:Config_SetSelectedEntry(value, 1, 1);
	self:Config_LoadValuesForSelectedEntry();
	CloseDropDownMenus();
end



function SpeakEasyGUI:Config_CategoryDropDown_Initialize(level)
	local buttonInfo = self:Main_GetReusableTable(1);
	buttonInfo.func = self.Config_CategoryDropDown_OnClick;
	buttonInfo.arg1 = self;
	buttonInfo.notCheckable = true;
	
	local addButton = UIDropDownMenu_AddButton;
	
	local speechCategories = self.SpeechCategories;
	
	
	if (level == 1) then
		local numCategories = #speechCategories;
		local errataNumToNum = self.LOCALIZATION.CATEGORY_ERRATA_NUM_TO_NUM;
		local tabTable;
		local tabNum = 1;
		local startErrataNum;
		
		while (tabNum <= numCategories) do
			tabTable = speechCategories[tabNum];
			
			if (tabTable.isErrata == true) then
				buttonInfo.hasArrow = true;
				buttonInfo.notClickable = true;
				buttonInfo.value = {};
				startErrataNum = (tabNum - 5);
				
				while (#buttonInfo.value < 10 and tabTable.isErrata == true) do
					buttonInfo.value[#buttonInfo.value + 1] = tabNum;
					tabNum = (tabNum + 1);
					tabTable = speechCategories[tabNum];
				end
				
				buttonInfo.text = errataNumToNum:format(startErrataNum, (tabNum - 6));
			else
				buttonInfo.disabled = nil;
				buttonInfo.hasArrow = nil;
				buttonInfo.notClickable = nil;
				buttonInfo.arg2 = tabNum;
				buttonInfo.text = tabTable.name;
				tabNum = (tabNum + 1);
			end
			
			addButton(buttonInfo, 1);
		end
		
		
	elseif (level == 2) then
		-- "this" is currently the arrow from dropdown 1.
		local errataList = this:GetParent().value;
		buttonInfo.func = self.Config_CategoryDropDown_OnClick;
		
		for index, tabNum in ipairs(errataList) do
			buttonInfo.arg2 = tabNum;
			buttonInfo.text = speechCategories[tabNum].name;
			
			addButton(buttonInfo, 2);
		end
	end
end


SpeakEasyGUI_CategoryDropDown.initialize = (function(level) SpeakEasyGUI:Config_CategoryDropDown_Initialize(level); end);




function SpeakEasyGUI:Config_MessageTypeDropDown_OnClick()
	local value = this.value;
	
	if (self.SelectedMessageType ~= 2 and value == 2) then
		SpeakEasyGUI_MessageEditBox:SetText(self.LOCALIZATION.FEEDBACK_BASIC_EMOTE_MESSAGEBOX_HELP);
	end
	
	self.SelectedMessageType = value;
	SpeakEasyGUI_MessageTypeDropDownText:SetText(self.MESSAGE_TYPE_LIST[value]);
end



function SpeakEasyGUI:Config_MessageTypeDropDown_Initialize()
	local buttonInfo = self:Main_GetReusableTable(2);
	buttonInfo.func = self.Config_MessageTypeDropDown_OnClick;
	buttonInfo.arg1 = self;
	buttonInfo.notCheckable = true;
	
	local addButton = UIDropDownMenu_AddButton;
	
	for index, messageType in ipairs(self.MESSAGE_TYPE_LIST) do
		buttonInfo.text = messageType;
		buttonInfo.value = index;
		addButton(buttonInfo);
	end
end


SpeakEasyGUI_MessageTypeDropDown.initialize = (function() SpeakEasyGUI:Config_MessageTypeDropDown_Initialize(); end);




function SpeakEasyGUI:Config_LanguageDropDown_OnClick()
	local value = this.value;
	self.SelectedLanguage = value;
	SpeakEasyGUI_LanguageDropDownText:SetText(self.LANGUAGE_LIST[value]);
end



function SpeakEasyGUI:Config_LanguageDropDown_Initialize()
	local buttonInfo = self:Main_GetReusableTable(1);
	buttonInfo.func = self.Config_LanguageDropDown_OnClick;
	buttonInfo.arg1 = self;
	buttonInfo.notCheckable = true;
	
	local addButton = UIDropDownMenu_AddButton;
	local language1 = GetLanguageByIndex(1);
	local language2 = GetLanguageByIndex(2);
	
	for index, language in ipairs(self.LANGUAGE_LIST) do
		if (language == language1 or language == language2) then
			buttonInfo.text = language;
			buttonInfo.value = index;
			addButton(buttonInfo);
		end
	end
end


SpeakEasyGUI_LanguageDropDown.initialize = (function() SpeakEasyGUI:Config_LanguageDropDown_Initialize(); end);




function SpeakEasyGUI:Config_PopulateDisplayTable()
	if (SpeakEasyGUI_DisplayFrame:IsShown()) then
		self.PopulateTableOnHide = true;
		return;
	end
	
	
	local localization = self.LOCALIZATION;
	local globalConfig = self.GlobalConfig;
	local config = self.LocalConfig;
	
	local tableinsert = self.TABLE_LIBRARY.insert;
	local tablesort = self.TABLE_LIBRARY.sort;
	
	local stringformat = self.STRING_LIBRARY.format;
	local stringsub = self.STRING_LIBRARY.sub;
	
	local pairs = pairs;
	
	
	self.SpeechCategories = {};
	local speechCategories = self.SpeechCategories;
	
	
	tableinsert(speechCategories, { name = localization.CATEGORY_HOTKEYS, storagePointer = globalConfig, subList = {} });
	local subList = speechCategories[1].subList;
	local hotkeyNum = localization.HOTKEY_NUMBER;
	
	for index = 1, self.MAX_ENTRIES do
		tableinsert(subList, stringformat(hotkeyNum, index));
	end
	
	
	tableinsert(speechCategories, { name = localization.CATEGORY_COMBAT_EVENTS, storagePointer = config.CombatEvents, subList = {} });
	subList = speechCategories[2].subList;
	
	for entryName in pairs(localization.COMBAT_EVENTS_LIST) do
		tableinsert(subList, entryName);
	end
	
	tablesort(subList);
	
	
	tableinsert(speechCategories, { name = localization.CATEGORY_MISC_EVENTS, storagePointer = config.MiscEvents, subList = {} });
	subList = speechCategories[3].subList;
	
	for entryName in pairs(localization.MISC_EVENTS_LIST) do
		tableinsert(subList, entryName);
	end
	
	tablesort(subList);
	
	
	tableinsert(speechCategories, { name = localization.CATEGORY_SELF_BUFFS, storagePointer = config.SelfBuffs, subList = {} });
	subList = speechCategories[4].subList;
	
	for entryName in pairs(config.SelfBuffs) do
		tableinsert(subList, entryName);
	end
	
	tablesort(subList);
	
	
	tableinsert(speechCategories, { name = localization.CATEGORY_SELF_DEBUFFS, storagePointer = config.SelfDebuffs, subList = {} });
	subList = speechCategories[5].subList;
	
	for entryName in pairs(config.SelfDebuffs) do
		tableinsert(subList, entryName);
	end
	
	tablesort(subList);
	
	
	local numConfigTabs = 5;
	local index = 0;
	local count = 0;
	
	for errataName in pairs(globalConfig.SpellErrata) do
		if (count == 0) then
			numConfigTabs = (numConfigTabs + 1);
			index = (index + 1);
			
			tableinsert(speechCategories, { name = stringformat(localization.CATEGORY_ERRATA_NUM, index), isErrata = true, subList = {} });
			subList = speechCategories[numConfigTabs].subList;
		end
		
		tableinsert(subList, errataName);
		
		count = (count + 1);
		if (count == 30) then
			tablesort(subList);
			count = 0;
		end
	end
	
	tablesort(subList);
	
	
	local stringmatch = self.STRING_LIBRARY.match;
	local getSpellTabInfo = GetSpellTabInfo;
	local isPassiveSpell = IsPassiveSpell;
	local getSpellName = GetSpellName;
	
	local tabName, _, offset, numSpells, spellName, startLoc, lastFoundName;
	
	for spellTab = 1, GetNumSpellTabs() do
		tabName, _, offset, numSpells = getSpellTabInfo(spellTab);
		
		tableinsert(speechCategories, { name = tabName, subList = {}, showSendAtStart = true });
		subList = speechCategories[spellTab + numConfigTabs].subList;
		
		numSpells = (numSpells + offset);
		offset = (offset + 1);
		
		for spellNum = offset, numSpells do
			
			if (not isPassiveSpell(spellNum, "spell")) then
				spellName = getSpellName(spellNum, "spell");
				
				-- Convert things like "Create Healthstone (Greater)" to just "Create Healthstone".
				startLoc = stringmatch(spellName, "()%s%(.+%)$");
				if (startLoc) then
					spellName = stringsub(spellName, 1, (startLoc - 1));
				end
				
				if (spellName ~= lastFoundName) then
					tableinsert(subList, spellName);
					lastFoundName = spellName;
				end
			end
			
		end
	end
	
	
	self:Config_SetSelectedEntry(1, 1, 1);
end

