

local SpeakEasyGUI = SpeakEasyGUI;




function SpeakEasyGUI:DataConversion_GlobalConfig_From2xxTo300(config)
	local mathceil = self.MATH_LIBRARY.ceil;
	local entryTable;
	
	-- UnitFactionGroup and GetDefaultLanguage both return incorrect values at the time when this function
	-- is most likely to run, so get the player's faction by race instead, since UnitRace is always accurate.
	local race = UnitRace("player");
	local defaultLanguage = (((race == "Orc" or race == "Scourge" or race == "Troll" or race == "Tauren" or race == "BloodElf") and 7) or 1);
	
	
	for index = 1, self.MAX_ENTRIES do
		entryTable = config[index];
		if (entryTable) then
			entryTable[2] = mathceil(entryTable[2] / 2);
			entryTable[3] = mathceil(entryTable[3] / 2);
			entryTable[5] = defaultLanguage;
		end
	end
end



function SpeakEasyGUI:DataConversion_LocalConfig_AllianceLanguageFix(config)
	local pairs = pairs;
	
	
	for buffName, buffTable in pairs(config.SelfBuffs) do for entryNum, entryTable in pairs(buffTable) do entryTable[5] = 1; end end
	
	for debuffName, debuffTable in pairs(config.SelfDebuffs) do for entryNum, entryTable in pairs(debuffTable) do entryTable[5] = 1; end end
	
	
	for combatName, combatTable in pairs(config.CombatEvents) do for entryNum, entryTable in pairs(combatTable) do entryTable[5] = 1; end end
	
	for miscName, miscTable in pairs(config.MiscEvents) do for entryNum, entryTable in pairs(miscTable) do entryTable[5] = 1; end end
	
	
	for spellName, spellTable in pairs(config) do
		if (spellName ~= "Version"
		  and spellName ~= "SelfBuffs" and spellName ~= "SelfDebuffs"
		  and spellName ~= "CombatEvents" and spellName ~= "MiscEvents"
		) then
			
			for entryNum, entryTable in pairs(spellTable) do
				entryTable[5] = 1;
			end
			
		end
	end
end



function SpeakEasyGUI:DataConversion_LocalConfig_From2xxTo300(config)
	local pairs = pairs;
	
	
	if (config.Items ~= nil) then
		for itemName, itemTable in pairs(config.Items) do
			if (self:Main_ValidateSpellAndAddToErrata(itemName)) then
				config[itemName] = itemTable;
			end
		end
		config.Items = nil;
	end
	
	
	-- UnitFactionGroup and GetDefaultLanguage both return incorrect values at the time when this function
	-- is most likely to run, so get the player's faction by race instead, since UnitRace is always accurate.
	local race = UnitRace("player");
	local defaultLanguage = (((race == "Orc" or race == "Scourge" or race == "Troll" or race == "Tauren" or race == "BloodElf") and 7) or 1);
	
	
	local mathceil = self.MATH_LIBRARY.ceil;
	
	if (config.SelfBuffs ~= nil) then
		for buffName, buffTable in pairs(config.SelfBuffs) do
			for entryNum, entryTable in pairs(buffTable) do
				entryTable[2] = mathceil(entryTable[2] / 2);
				entryTable[3] = mathceil(entryTable[3] / 2);
				entryTable[5] = defaultLanguage;
			end
		end
	end
	
	if (config.SelfDebuffs ~= nil) then
		for debuffName, debuffTable in pairs(config.SelfDebuffs) do
			for entryNum, entryTable in pairs(debuffTable) do
				entryTable[2] = mathceil(entryTable[2] / 2);
				entryTable[3] = mathceil(entryTable[3] / 2);
				entryTable[5] = defaultLanguage;
			end
		end
	end
	
	
	local stringsub = self.STRING_LIBRARY.sub;
	local varsToEvents = self.VARIABLES_TO_EVENTS;
	local newCombatEvents = {};
	local shortName;
	
	if (config.CombatEvents ~= nil) then
		for combatName, combatTable in pairs(config.CombatEvents) do
			shortName = stringsub(combatName, 8); -- Trim "COMBAT_" from, for example, "COMBAT_YOU_HIT".
			for entryNum, entryTable in pairs(combatTable) do
				entryTable[2] = mathceil(entryTable[2] / 2);
				entryTable[3] = mathceil(entryTable[3] / 2);
				entryTable[5] = defaultLanguage;
			end
			newCombatEvents[shortName] = combatTable;
		end
	end
	
	config.CombatEvents = newCombatEvents;
	
	
	if (config.MiscEvents ~= nil) then
		for miscName, miscTable in pairs(config.MiscEvents) do
			for entryNum, entryTable in pairs(miscTable) do
				entryTable[2] = mathceil(entryTable[2] / 2);
				entryTable[3] = mathceil(entryTable[3] / 2);
				entryTable[5] = defaultLanguage;
			end
		end
	end
	
	
	for spellName, spellTable in pairs(config) do
		if (spellName ~= "Version"
		  and spellName ~= "SelfBuffs" and spellName ~= "SelfDebuffs"
		  and spellName ~= "CombatEvents" and spellName ~= "MiscEvents"
		) then
			
			for entryNum, entryTable in pairs(spellTable) do
				entryTable[2] = mathceil(entryTable[2] / 2);
				entryTable[3] = mathceil(entryTable[3] / 2);
				entryTable[5] = defaultLanguage;
			end
			
		end
	end
end

