

local SpeakEasyGUI = SpeakEasyGUI;




function SpeakEasyGUI:SpecialEntries_ManageEventRegistration()
	local frame = SpeakEasyGUI_DisplayFrame;
	local reg = frame.RegisterEvent;
	
	
	frame:UnregisterAllEvents();
	reg(frame, "UNIT_SPELLCAST_SENT");
	reg(frame, "UNIT_SPELLCAST_SUCCEEDED");
	reg(frame, "PLAYER_ENTERING_WORLD");
	reg(frame, "LEARNED_SPELL_IN_TAB");
	
	
	local config = self.LocalConfig;
	
	
	if (next(config.SelfBuffs) ~= nil) then reg(frame, "CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS"); end
	
	if (next(config.SelfDebuffs) ~= nil) then reg(frame, "CHAT_MSG_SPELL_PERIODIC_SELF_DAMAGE"); end
	
	
	local variablesToEvents = self.VARIABLES_TO_EVENTS;
	local eventName;
	
	for varName in pairs(config.CombatEvents) do
		eventName = variablesToEvents[varName];
		if (eventName ~= nil) then
			reg(frame, eventName);
		end
	end
	
	for varName in pairs(config.MiscEvents) do
		eventName = variablesToEvents[varName];
		if (eventName ~= nil) then
			reg(frame, eventName);
		end
	end
end




function SpeakEasyGUI:SpecialEntries_GetCombatActionForChatEvent(event)
	for varName, eventName in pairs(self.VARIABLES_TO_EVENTS) do
		if (event == eventName) then
			return self.LocalConfig.CombatEvents[varName];
		end
	end
	
	return nil;
end




ReloadUI = function()
	local action = SpeakEasyGUI.LocalConfig.MiscEvents["RELOAD_UI"];
	
	if (action) then
		SpeakEasyGUI:Speech_ActivateValidEntriesForAction(action);
	end
	
	
	SpeakEasyGUI.Original_ReloadUI();
end




function SpeakEasyGUI:SpecialEntries_CheckPlayerHealthPercentage()
	if (UnitHealth("player") / UnitHealthMax("player") < 0.20) then
		
		if (self.HealthBelowTwenty == nil) then
			self.HealthBelowTwenty = true;
			return self.LocalConfig.MiscEvents["PLAYER_BELOW_TWENTY_HEALTH"];
		end
		
	else
		self.HealthBelowTwenty = nil;
	end
	
	return nil;
end




function SpeakEasyGUI:SpecialEntries_CheckPlayerManaPercentage()
	if (UnitMana("player") / UnitManaMax("player") < 0.20) then
		
		if (self.ManaBelowTwenty == nil) then
			self.ManaBelowTwenty = true;
			return self.LocalConfig.MiscEvents["PLAYER_BELOW_TWENTY_MANA"];
		end
		
	else
		self.ManaBelowTwenty = nil;
	end
	
	return nil;
end




function SpeakEasyGUI:SpecialEntries_CheckPlayerResurrectionState()
	local getPlayerBuffTexture = GetPlayerBuffTexture;
	
	for index = 1, 24 do
		if (getPlayerBuffTexture(index) == "Interface\\Icons\\Spell_Magic_LesserInvisibilty") then
			return nil;
		end
	end
	
	return self.LocalConfig.MiscEvents["PLAYER_RESURRECTION"];
end




function SpeakEasyGUI:SpecialEntries_GetActionFromBuffDebuffText(text, validityString, searchTable)
	local stringmatch = self.STRING_LIBRARY.match;
	
	if (stringmatch(text, validityString) ~= nil) then
		for entryName, entryTable in pairs(searchTable) do
			
			if (stringmatch(text, entryName) ~= nil) then
				return entryTable;
			end
			
		end
	end
	
	return nil;
end




function SpeakEasyGUI:SpecialEntries_AddSelfBuff(name)
	if (name == nil or self.LocalConfig.SelfBuffs[name] ~= nil) then return nil; end
	
	
	self.LocalConfig.SelfBuffs[name] = {
		[1] = {
			[1] = self.LOCALIZATION.FEEDBACK_ENTRY_MESSAGE_PROMPT;
			[2] = self.MIN_RANDOM;
			[3] = self.MAX_RANDOM;
			[4] = 8;
			[5] = 9;
		};
	};
	
	if (GetDefaultLanguage() == "Common") then
		self.LocalConfig.SelfBuffs[name][1][5] = 1;
	end
	
	self.TABLE_LIBRARY.insert(self.SpeechCategories[4].subList, name);
	
	
	self:Config_SetSelectedEntry(4, #self.SpeechCategories[4].subList, 1);
	self:Config_LoadValuesForSelectedEntry();
	
	return true;
end



function SpeakEasyGUI:SpecialEntries_RemoveSelfBuff(name)
	if (name == nil or self.LocalConfig.SelfBuffs[name] == nil) then return nil; end
	
	
	self.LocalConfig.SelfBuffs[name] = nil;
	
	for entryNum, entryName in pairs(self.SpeechCategories[4]) do
		if (entryName == name) then
			self.SpeechCategories[4][entryNum] = nil;
		end
	end
	
	
	if (self.CurrentCategoryNumber == 4) then
		self:Config_SetSelectedEntry(1, 1, 1);
		self:Config_LoadValuesForSelectedEntry();
	end
	
	return true;
end




function SpeakEasyGUI:SpecialEntries_AddSelfDebuff(name)
	if (name == nil or self.LocalConfig.SelfDebuffs[name] ~= nil) then return nil; end
	
	
	self.LocalConfig.SelfDebuffs[name] = {
		[1] = {
			[1] = self.LOCALIZATION.FEEDBACK_ENTRY_MESSAGE_PROMPT;
			[2] = self.MIN_RANDOM;
			[3] = self.MAX_RANDOM;
			[4] = 8;
			[5] = 9;
		};
	};
	
	if (GetDefaultLanguage() == "Common") then
		self.LocalConfig.SelfDebuffs[name][1][5] = 1;
	end
	
	self.TABLE_LIBRARY.insert(self.SpeechCategories[5].subList, name);
	
	
	self:Config_SetSelectedEntry(5, #self.SpeechCategories[5].subList, 1);
	self:Config_LoadValuesForSelectedEntry();
	
	return true;
end



function SpeakEasyGUI:SpecialEntries_RemoveSelfDebuff(name)
	if (name == nil or self.LocalConfig.SelfDebuffs[name] == nil) then return nil; end
	
	
	self.LocalConfig.SelfDebuffs[name] = nil;
	
	for entryNum, entryName in pairs(self.SpeechCategories[5]) do
		if (entryName == name) then
			self.SpeechCategories[5][entryNum] = nil;
		end
	end
	
	
	if (self.CurrentCategoryNumber == 5) then
		self:Config_SetSelectedEntry(1, 1, 1);
		self:Config_LoadValuesForSelectedEntry();
	end
	
	return true;
end




--[[
function SpeakEasyGUI:SpecialEntries_SetupCombatParsingTable()
	local combatParsing = self.COMBATEVENT_PARSING_BY_CHATEVENT;
	local stringgsub = self.STRING_LIBRARY.gsub;
	
	
	local spellSelfDamage = combatParsing["CHAT_MSG_SPELL_SELF_DAMAGE"]; -- Spells and abilities cast BY, not AT, the player.
	
	spellSelfDamage[1] = stringgsub((stringgsub(SPELLLOGSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Ability hit enemy
	spellSelfDamage[2] = stringgsub((stringgsub(SPELLLOGSCHOOLSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Spell hit enemy
	
	spellSelfDamage[3] = stringgsub((stringgsub(SPELLLOGCRITSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Ability crit enemy
	spellSelfDamage[4] = stringgsub((stringgsub(SPELLLOGCRITSCHOOLSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Spell crit enemy
	
	spellSelfDamage[5] = stringgsub(SPELLDODGEDSELFOTHER, "%%s", "%%w+"); -- Enemy dodged special
	spellSelfDamage[6] = stringgsub(SPELLRESISTSELFOTHER, "%%s", "%%w+"); -- Enemy resisted special
	spellSelfDamage[7] = stringgsub(MISSEDSELFOTHER, "%%s", "%%w+"); -- Player missed special
	spellSelfDamage[8] = stringgsub(SPELLPARRIEDSELFOTHER, "%%s", "%%w+"); -- Enemy parried special
	spellSelfDamage[9] = stringgsub(SPELLBLOCKEDSELFOTHER, "%%s", "%%w+"); -- Enemy blocked special
	
	
	local spellSelfBuff = combatParsing["CHAT_MSG_SPELL_SELF_BUFF"]; -- Heals cast BY, not ON, the player.
	
	spellSelfBuff[1] = stringgsub((stringgsub(HEALEDSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Normal heal
	spellSelfBuff[2] = stringgsub((stringgsub(HEALEDCRITSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Crit heal
	
	
	local selfHits = combatParsing["CHAT_MSG_COMBAT_SELF_HITS"]; -- Normal melee attacks made BY, not AGAINST, the player, which hit.
	
	selfHits[1] = stringgsub((stringgsub(COMBATHITSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Standard hit enemy
	selfHits[2] = stringgsub((stringgsub(COMBATHITCRITSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Standard crit enemy
	
	-- Players do elemental melee damage so very rarely that the parsing costs of the following two entries just aren't worthwhile.
	--selfHits[3] = stringgsub((stringgsub(COMBATHITSCHOOLSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Elemental hit enemy
	--selfHits[4] = stringgsub((stringgsub(COMBATHITCRITSCHOOLSELFOTHER, "%%s", "%%w+")), "%%d", "%%d+"); -- Elemental crit enemy
	
	
	local selfMisses = combatParsing["CHAT_MSG_COMBAT_SELF_MISSES"]; -- Normal melee attacks made BY, not AGAINST, the player, which miss.
	
	selfMisses[1] = stringgsub(VSDODGESELFOTHER, "%%s", "%%w+"); -- Enemy dodged standard
	selfMisses[2] = stringgsub(MISSEDSELFOTHER, "%%s", "%%w+"); -- Player missed standard
	selfMisses[3] = stringgsub(VSPARRYSELFOTHER, "%%s", "%%w+"); -- Enemy parried standard
	selfMisses[4] = stringgsub(VSBLOCKSELFOTHER, "%%s", "%%w+"); -- Enemy blocked standard
	
	
	local creatureVsSelfDamage = combatParsing["CHAT_MSG_COMBAT_CREATURE_VS_SELF_DAMAGE"]; -- Spells and abilities cast AT, not BY, the player.
	
	creatureVsSelfDamage[1] = stringgsub((stringgsub(SPELLLOGOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Ability hit player
	creatureVsSelfDamage[2] = stringgsub((stringgsub(SPELLLOGCRITSCHOOLOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Spell hit player
	
	creatureVsSelfDamage[3] = stringgsub((stringgsub(SPELLLOGCRITOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Ability crit player
	creatureVsSelfDamage[4] = stringgsub((stringgsub(SPELLLOGCRITSCHOOLOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Spell crit player
	
	creatureVsSelfDamage[5] = stringgsub(SPELLDODGEDOTHERSELF, "%%s", "%%w+"); -- Player dodged special
	creatureVsSelfDamage[6] = stringgsub(SPELLRESISTOTHERSELF, "%%s", "%%w+"); -- Player resisted special
	creatureVsSelfDamage[7] = stringgsub(MISSEDOTHERSELF, "%%s", "%%w+"); -- Enemy missed special
	creatureVsSelfDamage[8] = stringgsub(SPELLPARRIEDOTHERSELF, "%%s", "%%w+"); -- Player parried special
	creatureVsSelfDamage[9] = stringgsub(SPELLBLOCKEDOTHERSELF, "%%s", "%%w+"); -- Player blocked special
	
	
	local creatureVsSelfHits = combatParsing["CHAT_MSG_COMBAT_CREATURE_VS_SELF_HITS"]; -- Normal melee attacks made AGAINST, not BY, the player, which hit.
	
	creatureVsSelfHits[1] = stringgsub((stringgsub(COMBATHITOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Standard hit player
	creatureVsSelfHits[2] = stringgsub((stringgsub(COMBATHITCRITOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Standard crit player
	
	creatureVsSelfHits[3] = stringgsub((stringgsub(COMBATHITSCHOOLOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Elemental hit player
	creatureVsSelfHits[4] = stringgsub((stringgsub(COMBATHITCRITSCHOOLOTHERSELF, "%%s", "%%w+")), "%%d", "%%d+"); -- Elemental crit player
	
	
	local creatureVsSelfMisses = combatParsing["CHAT_MSG_COMBAT_CREATURE_VS_SELF_MISSES"];  -- Normal melee attacks made AGAINST, not BY, the player, which miss.
	
	creatureVsSelfMisses[1] = stringgsub(VSDODGEOTHERSELF, "%%s", "%%w+"); -- Player dodged standard
	creatureVsSelfMisses[2] = stringgsub(MISSEDOTHERSELF, "%%s", "%%w+"); -- Enemy missed standard
	creatureVsSelfMisses[3] = stringgsub(VSPARRYOTHERSELF, "%%s", "%%w+"); -- Player parried standard
	creatureVsSelfMisses[4] = stringgsub(VSBLOCKOTHERSELF, "%%s", "%%w+"); -- Player blocked standard
end
]]--

