

local SpeakEasyGUI = SpeakEasyGUI;




function SpeakEasyGUI:Speech_SendTextForEntry(entryTable)
	local messageType = entryTable[4];
	
	
	if (messageType == 1) then -- Channel
		local chanNum = GetChannelName(entryTable[6]);
		if (chanNum) then
			SendChatMessage(entryTable[1], "CHANNEL", nil, chanNum);
		end
		
		
	elseif (messageType == 9) then -- Whisper
		if (entryTable[6] == "%t") then
			if (UnitExists("target") and UnitIsFriend("player", "target")) then
				local whisperName, realmName = UnitName("target");
				if (realmName) then
					SendChatMessage(entryTable[1], "WHISPER", self.LANGUAGE_LIST[entryTable[5]], whisperName.."-"..realmName);
				else
					SendChatMessage(entryTable[1], "WHISPER", self.LANGUAGE_LIST[entryTable[5]], whisperName);
				end
			end
		else
			SendChatMessage(entryTable[1], "WHISPER", self.LANGUAGE_LIST[entryTable[5]], entryTable[6]);
		end
		
		
	elseif (messageType == 2) then -- Basic Emote
		DoEmote(entryTable[1], entryTable[6]);
		
		
	elseif (messageType == 3) then -- Custom Emote
		SendChatMessage(entryTable[1], "EMOTE");
		
		
	elseif (messageType == 6) then -- Party
		if (GetNumPartyMembers() > 0) then
			SendChatMessage(entryTable[1], "PARTY", self.LANGUAGE_LIST[entryTable[5]]);
		end
		
		
	elseif (messageType == 7) then -- Raid
		if (GetNumRaidMembers() > 0) then
			SendChatMessage(entryTable[1], "RAID", self.LANGUAGE_LIST[entryTable[5]]);
		end
		
		
	elseif (messageType == 11) then -- Raid Warning
		if (GetNumRaidMembers() > 0 and (IsRaidOfficer() or IsRaidLeader())) then
			SendChatMessage(entryTable[1], "RAID_WARNING", self.LANGUAGE_LIST[entryTable[5]]);
		end
		
		
	elseif (messageType == 12) then -- Battleground
		if (GetNumRaidMembers() > 0) then
			SendChatMessage(entryTable[1], "BATTLEGROUND", self.LANGUAGE_LIST[entryTable[5]]);
		end
		
		
	else -- Everything else
		SendChatMessage(entryTable[1], self.MESSAGE_TYPE_LIST[messageType], self.LANGUAGE_LIST[entryTable[5]]);
	end
end




function SpeakEasyGUI.Speech_DelayedEntryFrame_OnUpdate(frame, elapsed)
	frame.elapsed = (frame.elapsed + elapsed);
	
	if (frame.elapsed > frame.delay) then
		SpeakEasyGUI:Speech_SendTextForEntry(frame.entryTable);
		
		frame.elapsed = nil;
		frame.delay = nil;
		frame.entryTable = nil;
		frame:Hide();
	end
end




function SpeakEasyGUI:Speech_QueueDelayedEntry(entryTable)
	local getglobal = getglobal;
	local index = 1;
	local currFrame;
	local freeFrame;
	
	while (not freeFrame) do
		currFrame = getglobal("SpeakEasyGUI_DelayedEntryFrame_"..index);
		
		if (not currFrame) then
			freeFrame = CreateFrame("Frame", "SpeakEasyGUI_DelayedEntryFrame_"..index, WorldFrame);
			freeFrame:Hide();
			freeFrame:SetScript("OnUpdate", self.Speech_DelayedEntryFrame_OnUpdate);
			
		elseif (not currFrame:IsShown()) then
			freeFrame = currFrame;
			
		else
			index = (index + 1);
		end
	end
	
	freeFrame.elapsed = 0.0;
	freeFrame.delay = entryTable[10];
	freeFrame.entryTable = entryTable;
	freeFrame:Show();
end




function SpeakEasyGUI:Speech_ActivateValidEntriesForAction(actionTable, isStart)
	if (not actionTable or self.LocalConfig.Silence) then return; end
	
	
	local randomNum = self.MATH_LIBRARY.random(self.MIN_RANDOM, self.MAX_RANDOM);
	
	local entryTable;
	
	
	for entryNum = 1, self.MAX_ENTRIES do
		entryTable = actionTable[entryNum];
		
		if (entryTable and isStart == entryTable[8]) then
			
			if (entryTable[9]) then
				-- If this entry was sent the previous time the action was triggered, clear the flag that says so, so it can be sent again next time.
				entryTable[9] = nil;
				
				
			elseif (randomNum >= entryTable[2] and randomNum <= entryTable[3]) then
				if (entryTable[10]) then
					self:Speech_QueueDelayedEntry(entryTable);
				else
					self:Speech_SendTextForEntry(entryTable);
				end
				
				if (entryTable[7]) then
					-- If this entry is not allowed to be sent twice in a row, set the flag value to indicate that it should be skipped next time this action is processed.
					entryTable[9] = true;
				else
					entryTable[9] = nil;
				end
			end
			
		end
	end
end




function SpeakEasyGUI:Speech_SpeakByGlobalNumber(keyNum)
	-- Some additional validity checks and feedback are placed in this method, because the user would obviously want to know if their macro or keybinding won't work because of a silent error.
	
	keyNum = tonumber(keyNum);
	
	if (keyNum ~= nil and keyNum > 0 and keyNum < self.MAX_ENTRIES) then
		local entry = self.GlobalConfig[self.LOCALIZATION.HOTKEY_NUMBER:format(keyNum)];
		
		if (entry ~= nil) then
			self:Speech_ActivateValidEntriesForAction(entry, nil);
		else
			DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.HOTKEY_BLANK_ENTRY:format(keyNum), 0.0, 0.8, 1.0);
		end
		
	else
		DEFAULT_CHAT_FRAME:AddMessage(self.LOCALIZATION.HOTKEY_INVALID_ARGUMENT:format(type(keyNum), tostring(keyNum)), 0.0, 0.8, 1.0);
	end
end


-- Backwards compatibility.
SpeakEasyGUI.Main_SpeakByHotkey = SpeakEasyGUI.Speech_SpeakByGlobalNumber;



-- Create this wrapper for use in macros, to allow the user to conserve space in the limited 255 characters.
SEGUI = (function(keyNum) SpeakEasyGUI:Speech_SpeakByGlobalNumber(keyNum); end);

